/*
  Scoring system for a CTF
*/ 

#include "lib\common.h" ;

VAR_DEFAULT(CTF_reverseScore,false) ;
VAR_DEFAULT(CTF_scoreToWin,100000) ;
VAR_DEFAULT(CTF_hudNoScore,false) ;
VAR_DEFAULT(CTF_hudNoObjectives,false) ;
VAR_DEFAULT(CTF_hudNoClock,false) ;

/* public variable used by both server and clients */
scoreHint="No score" ;
scoreArray=[] ;
dominationWin="" ;

if (isServer) then {
     initScore ={
	  east_points =0 ;
	  west_points =0 ;
     } ;
     
     flagHeld = {
	  private ["_side"];
	  _side = ARG0 ;
	  call compile format ["%1_points=%1_points+1",_side];
     } ;

     scoreString = {
	  private ["_r"] ;
	  if (CTF_reverseScore) then {
	       _r = format ["%1\nEAST %2 points\nWEST %3 points\n",
		       _this select 0,
		       CTF_scoreToWin-west_points, CTF_scoreToWin-east_points] ;
	  } else {
	       _r = format ["%1\nEAST %2 points\nWEST %3 points\n",
		       _this select 0,
		       east_points, west_points] ;
	  } ;
	  if (CTF_hudNoScore) then {_r = "";} ;
	  _r ;
     } ;
     updateScoreHint ={
     } ;

     updateScoreStatus ={
	  east_points = east_points + ARG0 ;
	  west_points = west_points + ARG1 ;
	  if (CTF_reverseScore) then {
	       scoreArray = [CTF_scoreToWin-west_points,CTF_scoreToWin-east_points] + ARG2 ;
	  } else {
	       scoreArray = [east_points,west_points] + ARG2 ;
	  } ;
	  //check to see if someone has won
	  if ((east_points >= CTF_scoreToWin) or (west_points >= CTF_scoreToWin)) then {
	       forceGameOver=true ;
	  } ;
	  publicVariable "scoreArray" ;
     } ;
     
     objectiveTaken={
	  private ["_flgname","_side","_s"] ;
	  _flgname =ARG0 ;
	  _side  =ARG1;
	  if (_side == "neutral") then {_s = format ["%1 is now neutral",_flgname] ;}
	  else {
	       _s = format ["%1 belongs to the %2 side", _flgname,_side];
	  };
	  [_s,"flagTaken"] call announce;
     } ;

     gameOverScore={
	  private ["_res","_r2"];
	  _res = "The game is a draw" ;
	  if (dominationWin == "") then {
	       if (east_points > west_points) then {  _res = "East wins !" ;} ;
	       if (west_points > east_points) then {  _res = "West wins !" ;} ;
	       if ((eastInPlay<=0)  and (westInPlay>0)) then {_res ="West have won by killing all east players";};
	       if ((westInPlay<=0)  and (eastInPlay>0)) then {_res ="East have won by killing all west players";};
	       
	       _r2 = [""] call scoreString ;
	  } else {
	       _r2 = "" ;
	       _res = format["%1 won by capturing all objectives",dominationWin] ;
	  } ;
	  
	  [format ["GAME OVER\n\n%1\n\n%2",_res,_r2] ] call announceCenter ;

     };

     allObjectivesTaken={
	  private ["_winner"] ;
	  _winner = ARG0;
	  if (_winner == "east") then {dominationWin="East"} ;
	  if (_winner == "west") then {dominationWin="West"} ;
	  forceGameOver=true ;
     }

} ;

objRegions=[] ;

if (not isNull player) then {
     hudTask={
	  private ["_t","_i","_col","_pre","_lastObj","_thisObj","_hudlines","_tsk","_n","_objectiveList"];
	  waituntil {gameStarted} ;
	  _objectiveList=RECEIVE(ObjectiveList) ;
	  //remove hidden objectives
	  _t = [] ;
	  {
	       if (not VAR(_x,hidden)) then { _t = _t + [_x] ; } ;
	  } foreach _objectiveList ;
	  _objectiveList=_t ;

	  _hudlines=1 ;
	  if (not CTF_hudNoObjectives) then {
	       _hudlines = 1+ _hudlines + count _objectiveList ;
	  };
	  if (not CTF_hudNoScore) then {
	       _hudlines = _hudlines + 2 ;
	  } ;
	  if (not CTF_hudNoClock) then {
	       _hudlines = 1+ _hudlines  ;
	  };
	  [_hudlines] call hudInit ;
	  //nasty hack to make sure scorearray is properly sized
	  scoreArray set[count _objectiveList+2,0] ;
	  {
	       //pos size dir shape
		    _n = VAR(_x,objName) ;

		    _tsk =  player createSimpleTask [_n] ;
		    _tsk setSimpleTaskDescription [_n,_n,_n] ;
		    _tsk setSimpleTaskDestination VAR(_x,center) ;
		    _tsk setTaskState "CREATED" ;
	       
	  } foreach _objectiveList;
	  
	  _lastObj = "you have left the objective"  ;
	  while {gameState=="playing"} do {
	       _thisObj = "" ;
	       if (not CTF_hudNoObjectives) then {
		    for "_i" from 0 to (count _objectiveList -1) do {
			 _pre = "  " ;
			 if ([EL(_objectiveList,_i),position player] call insideRegion) then {
			      _pre = "==" ;
			      _thisObj = "You are at " + VAR(EL(_objectiveList,_i),objName) ;
			 } ;
			 _t =format ["%1%2: %3%4",_pre,VAR(EL(_objectiveList,_i),objName),EL(scoreArray,_i+2),_pre] ; 
			 [_i,_t] call hudSetLine ;
			 _col = "" ;
			 switch (EL(scoreArray,_i+2)) do {
			 case "west" : {_col="ColorBlue"; };
			 case "east" : {_col="ColorRed"; };
			 case "neutral" : {_col="ColorWhite"; };
			 } ;
			 [_i,_col] call hudSetLineColour ;
		    } ;
	       } ;
	       if (not CTF_hudNoScore) then {
		    [-3,format ["East: %1",EL(scoreArray,0)] ] call hudSetLine ;
		    [-2,format ["West: %1",EL(scoreArray,1)] ] call hudSetLine ;
	       } ;
	       if (not CTF_hudNoClock) then {
		    [-1,[] call TimeLeftString] call hudSetLine ;
	       } ;
	       scoreHint = [] call hudStringify ;
	       if (_thisObj != _lastObj) then {
		    hint _thisObj ;
		    _lastObj=_thisObj ;
	       } ;
	       sleep 1 ;
	  } ;
	  call hudHide ;
     } ;
};
     
     
